import string
import random
import re

from enum import Enum

MONTHS = {
    "января": 1,
    "февраля": 2,
    "марта": 3,
    "апреля": 4,
    "мая": 5,
    "июня": 6,
    "июля": 7,
    "августа": 8,
    "сентября": 9,
    "октября": 10,
    "ноября": 11,
    "декабря": 12
}

MONTHS1 = {
    "январь": 1,
    "февраль": 2,
    "март": 3,
    "апрель": 4,
    "май": 5,
    "июнь": 6,
    "июль": 7,
    "август": 8,
    "сентябрь": 9,
    "октябрь": 10,
    "ноябрь": 11,
    "декабрь": 12
}

def raising_lots_time(response: str) -> int:
    """
    Парсинг времяни поднятия лотов.

    :param response: Json ответ на запрос к сайту.

    :return: Примерное время до следующего запроса (в секундах).
    """
    if response == "Подождите секунду.":
        return 2
    elif response == "Подождите минуту.":
        return 61
    elif response == "Подождите час.":
        return 3601
    elif "сек" in response:
        response = response.split()
        return int(response[1])
    elif "мин" in response:
        response = response.split()
        return (int(response[1])-1) * 60
    elif "час" in response:
        response = response.split()
        return (int(response[1])) * 3600
    else:
        return 10
    
class RegularExpressions(object):
    """
    В данном классе хранятся скомпилированные тексты, описывающие событие на фанпей
    Класс является singleton'ом.
    """
    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, "instance"):
            setattr(cls, "instance", super(RegularExpressions, cls).__new__(cls))
        return getattr(cls, "instance")

    def __init__(self):
        self.NEW_ORDER1 = re.compile(r"Покупатель [a-zA-Z0-9]+ оплатил заказ #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение об оплате заказа.
        Советуется использовать вместе с NEW_ORDER2
        """

        self.NEW_ORDER2 = re.compile(r"[a-zA-Z0-9]+, не забудьте потом нажать кнопку "
                                           r"«Подтвердить выполнение заказа»\.")
        """
        Регулярное выражение, описывающее сообщение об оплате заказа (2).
        Советуется использовать вместе с NEW_ORDER1
        """

        self.ORDER_CONFIRMED = re.compile(r"Покупатель [a-zA-Z0-9]+ подтвердил успешное выполнение "
                                          r"заказа #[A-Z0-9]{8} и отправил деньги продавцу [a-zA-Z0-9]+\.")
        """
        Регулярное выражение, описывающее сообщение о подтверждении выполнения заказа.
        """

        self.NEW_REVIEW = re.compile(r"Покупатель [a-zA-Z0-9]+ написал отзыв к заказу #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение о новом отзыве.
        """

        self.EDIT_REVIEW = re.compile(r"Покупатель [a-zA-Z0-9]+ изменил отзыв к заказу #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение об изменении отзыва.
        """

        self.DELETE_REVIEW = re.compile(r"Покупатель [a-zA-Z0-9]+ удалил отзыв к заказу #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение об удалении отзыва.
        """

        self.NEW_ANSWER_REVIEW = re.compile(r"Продавец [a-zA-Z0-9]+ ответил на отзыв к заказу #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение о новом ответе на отзыв.
        """

        self.EDIT_ANSWER_REVIEW = re.compile(r"Продавец [a-zA-Z0-9]+ изменил ответ на отзыв к "
                                                  r"заказу #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение об изменении ответа на отзыв.
        """

        self.DELETE_ANSWER_REVIEW = re.compile(r"Продавец [a-zA-Z0-9]+ удалил ответ на отзыв к заказу "
                                                  r"#[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение об удалении ответа на отзыв.
        """

        self.REOPENED_ORDER = re.compile(r"Заказ #[A-Z0-9]{8} открыт повторно\.")
        """
        Регулярное выражение, описывающее сообщение о повтором открытии заказа.
        """

        self.REFUND_ORDER = re.compile(r"Продавец [a-zA-Z0-9]+ вернул деньги покупателю [a-zA-Z0-9]+ "
                                 r"по заказу #[A-Z0-9]{8}\.")
        """
        Регулярное выражение, описывающее сообщение о возврате денежных средств.
        """

        self.PARTIAL_REFUND_ORDER = re.compile(r"Часть средств по заказу #[A-Z0-9]{8} возвращена покупателю\.")
        """
        Регулярное выражение, описывающее сообщение частичном о возврате денежных средств.
        """

        self.CONFIRMED_ORDER_BY_ADMIN = re.compile(r"Администратор [a-zA-Z0-9]+ подтвердил успешное выполнение "
                                                   r"заказа #[A-Z0-9]{8} и отправил деньги продавцу [a-zA-Z0-9]+\.")
        """
        Регулярное выражение, описывающее сообщение о подтверждении выполнения заказа администратором.
        """

        self.ORDER_ID = re.compile(r"#[A-Z0-9]{8}")
        """
        Регулярное выражение, описывающее ID заказа.
        """

        self.ORDER_DATE = re.compile(r"\d{1,2} [а-я]+, \d{1,2}:\d{1,2}")
        """
        Регулярное выражение, описывающее дату заказа в формате <ДД месяца, ЧЧ:ММ>.
        """

        self.FULL_ORDER_DATE = re.compile(r"\d{1,2} [а-я]+ \d{4}, \d{1,2}:\d{1,2}")
        """
        Регулярное выражение, описывающее дату заказа в формате <ДД месяца ГГГГ, ЧЧ:ММ>.
        """

        self.DISCORD = "Вы можете перейти в Discord. " \
                       "Внимание: общение за пределами сервера FunPay считается нарушением правил."
        """
        Текст сообщения о предложении перехода в Discord.
        """

        self.PRODUCTS_AMOUNT = re.compile(r"\d+ шт\.")
        """
        Регулярное выражение, описывающее запись кол-ва товаров в заказе.
        """

class Currency(Enum):
    """
    В данном классе перечислены все виды валют баланса.
    """
    USD = 0
    """Доллар"""
    RUB = 1
    """Рубль"""
    EUR = 2
    """Евро"""

class WithdrawingМethods(Enum):
    """
    В этом классе перечислены все методы для вывода денег с баланса.
    """
    CARD_RUB = 0
    """Банковская карта в рублях."""
    CARD_USD = 1
    """Банковская карта в долларах."""
    CARD_EUR = 2
    """Банковская карта в евро."""
    YOUMONEY = 3
    """ЮMoney."""
    WEBMONEY = 4
    """WebMoney WMZ."""
    BINANCE = 5
    """Binance Pay."""
    TRC = 6
    """USDT TRC20."""


